@@ -12,6 +12,7 @@ from traitlets import Bool
 from traitlets import default
 from traitlets import Dict
 from traitlets import HasTraits
+from traitlets import Instance
 from traitlets import List
 from traitlets import Unicode
 from traitlets.config import Config
@@ -103,7 +104,7 @@ class ExtensionAppJinjaMixin(HasTraits):
             loader=FileSystemLoader(self.template_paths),
             extensions=["jinja2.ext.i18n"],
             autoescape=True,
-            **self.jinja2_options
+            **self.jinja2_options,
         )
 
         # Add the jinja2 environment for this extension to the tornado settings.
@@ -162,6 +163,12 @@ class ExtensionApp(JupyterApp):
     def _default_open_browser(self):
         return self.serverapp.config["ServerApp"].get("open_browser", True)
 
+    @property
+    def config_file_paths(self):
+        """Look on the same path as our parent for config files"""
+        # rely on parent serverapp, which should control all config loading
+        return self.serverapp.config_file_paths
+
     # The extension name used to name the jupyter config
     # file, jupyter_{name}_config.
     # This should also match the jupyter subcommand used to launch
@@ -200,7 +207,21 @@ class ExtensionApp(JupyterApp):
     ]
 
     # A ServerApp is not defined yet, but will be initialized below.
-    serverapp = None
+    serverapp = Instance(ServerApp)
+
+    @default("serverapp")
+    def _default_serverapp(self):
+        # load the current global instance, if any
+        if ServerApp.initialized():
+            try:
+                return ServerApp.instance()
+            except Exception:
+                # error retrieving instance, e.g. MultipleInstanceError
+                pass
+
+        # serverapp accessed before it was defined,
+        # declare an empty one
+        return ServerApp()
 
     _log_formatter_cls = LogFormatter
 
