@@ -1,23 +1,21 @@
 # coding: utf-8
 """Utilities for installing extensions"""
-
 # Copyright (c) Jupyter Development Team.
 # Distributed under the terms of the Modified BSD License.
-
 import os
 import sys
+
+from jupyter_core.application import JupyterApp
+from jupyter_core.paths import ENV_CONFIG_PATH
+from jupyter_core.paths import jupyter_config_dir
+from jupyter_core.paths import SYSTEM_CONFIG_PATH
 from tornado.log import LogFormatter
 from traitlets import Bool
 
-from jupyter_core.application import JupyterApp
-from jupyter_core.paths import (
-    jupyter_config_dir,
-    ENV_CONFIG_PATH,
-    SYSTEM_CONFIG_PATH
-)
 from jupyter_server._version import __version__
 from jupyter_server.extension.config import ExtensionConfigManager
-from jupyter_server.extension.manager import ExtensionManager, ExtensionPackage
+from jupyter_server.extension.manager import ExtensionManager
+from jupyter_server.extension.manager import ExtensionPackage
 
 
 def _get_config_dir(user=False, sys_prefix=False):
@@ -71,34 +69,49 @@ def _get_extmanager_for_context(write_dir="jupyter_server_config.d", user=False,
 class ArgumentConflict(ValueError):
     pass
 
+
 _base_flags = {}
 _base_flags.update(JupyterApp.flags)
 _base_flags.pop("y", None)
 _base_flags.pop("generate-config", None)
-_base_flags.update({
-    "user" : ({
-        "BaseExtensionApp" : {
-            "user" : True,
-        }}, "Apply the operation only for the given user"
-    ),
-    "system" : ({
-        "BaseExtensionApp" : {
-            "user" : False,
-            "sys_prefix": False,
-        }}, "Apply the operation system-wide"
-    ),
-    "sys-prefix" : ({
-        "BaseExtensionApp" : {
-            "sys_prefix" : True,
-        }}, "Use sys.prefix as the prefix for installing extensions (for environments, packaging)"
-    ),
-    "py" : ({
-        "BaseExtensionApp" : {
-            "python" : True,
-        }}, "Install from a Python package"
-    )
-})
-_base_flags['python'] = _base_flags['py']
+_base_flags.update(
+    {
+        "user": (
+            {
+                "BaseExtensionApp": {
+                    "user": True,
+                }
+            },
+            "Apply the operation only for the given user",
+        ),
+        "system": (
+            {
+                "BaseExtensionApp": {
+                    "user": False,
+                    "sys_prefix": False,
+                }
+            },
+            "Apply the operation system-wide",
+        ),
+        "sys-prefix": (
+            {
+                "BaseExtensionApp": {
+                    "sys_prefix": True,
+                }
+            },
+            "Use sys.prefix as the prefix for installing extensions (for environments, packaging)",
+        ),
+        "py": (
+            {
+                "BaseExtensionApp": {
+                    "python": True,
+                }
+            },
+            "Install from a Python package",
+        ),
+    }
+)
+_base_flags["python"] = _base_flags["py"]
 
 _base_aliases = {}
 _base_aliases.update(JupyterApp.aliases)
@@ -106,6 +119,7 @@ _base_aliases.update(JupyterApp.aliases)
 
 class BaseExtensionApp(JupyterApp):
     """Base extension installer app"""
+
     _log_formatter_cls = LogFormatter
     flags = _base_flags
     aliases = _base_aliases
@@ -126,10 +140,10 @@ class BaseExtensionApp(JupyterApp):
 
 # Constants for pretty print extension listing function.
 # Window doesn't support coloring in the commandline
-GREEN_ENABLED = '\033[32menabled\033[0m' if os.name != 'nt' else 'enabled'
-RED_DISABLED = '\033[31mdisabled\033[0m' if os.name != 'nt' else 'disabled'
-GREEN_OK = '\033[32mOK\033[0m' if os.name != 'nt' else 'ok'
-RED_X = '\033[31m X\033[0m' if os.name != 'nt' else ' X'
+GREEN_ENABLED = "\033[32menabled\033[0m" if os.name != "nt" else "enabled"
+RED_DISABLED = "\033[31mdisabled\033[0m" if os.name != "nt" else "disabled"
+GREEN_OK = "\033[32mOK\033[0m" if os.name != "nt" else "ok"
+RED_X = "\033[31m X\033[0m" if os.name != "nt" else " X"
 
 # ------------------------------------------------------------------------------
 # Public API
@@ -137,11 +151,7 @@ RED_X = '\033[31m X\033[0m' if os.name != 'nt' else ' X'
 
 
 def toggle_server_extension_python(
-    import_name,
-    enabled=None,
-    parent=None,
-    user=False,
-    sys_prefix=True
+    import_name, enabled=None, parent=None, user=False, sys_prefix=True
 ):
     """Toggle the boolean setting for a given server extension
     in a Jupyter config file.
@@ -150,13 +160,14 @@ def toggle_server_extension_python(
     config_dir = _get_config_dir(user=user, sys_prefix=sys_prefix)
     manager = ExtensionConfigManager(
         read_config_path=[config_dir],
-        write_config_dir=os.path.join(config_dir, "jupyter_server_config.d")
+        write_config_dir=os.path.join(config_dir, "jupyter_server_config.d"),
     )
     if enabled:
         manager.enable(import_name)
     else:
         manager.disable(import_name)
 
+
 # ----------------------------------------------------------------------
 # Applications
 # ----------------------------------------------------------------------
@@ -165,42 +176,57 @@ flags = {}
 flags.update(BaseExtensionApp.flags)
 flags.pop("y", None)
 flags.pop("generate-config", None)
-flags.update({
-    "user" : ({
-        "ToggleServerExtensionApp" : {
-            "user" : True,
-        }}, "Perform the operation for the current user"
-    ),
-    "system" : ({
-        "ToggleServerExtensionApp" : {
-            "user" : False,
-            "sys_prefix": False,
-        }}, "Perform the operation system-wide"
-    ),
-    "sys-prefix" : ({
-        "ToggleServerExtensionApp" : {
-            "sys_prefix" : True,
-        }}, "Use sys.prefix as the prefix for installing server extensions"
-    ),
-    "py" : ({
-        "ToggleServerExtensionApp" : {
-            "python" : True,
-        }}, "Install from a Python package"
-    ),
-})
-flags['python'] = flags['py']
+flags.update(
+    {
+        "user": (
+            {
+                "ToggleServerExtensionApp": {
+                    "user": True,
+                }
+            },
+            "Perform the operation for the current user",
+        ),
+        "system": (
+            {
+                "ToggleServerExtensionApp": {
+                    "user": False,
+                    "sys_prefix": False,
+                }
+            },
+            "Perform the operation system-wide",
+        ),
+        "sys-prefix": (
+            {
+                "ToggleServerExtensionApp": {
+                    "sys_prefix": True,
+                }
+            },
+            "Use sys.prefix as the prefix for installing server extensions",
+        ),
+        "py": (
+            {
+                "ToggleServerExtensionApp": {
+                    "python": True,
+                }
+            },
+            "Install from a Python package",
+        ),
+    }
+)
+flags["python"] = flags["py"]
 
 
 class ToggleServerExtensionApp(BaseExtensionApp):
     """A base class for enabling/disabling extensions"""
+
     name = "jupyter server extension enable/disable"
     description = "Enable/disable a server extension using frontend configuration files."
 
     flags = flags
 
     _toggle_value = Bool()
-    _toggle_pre_message = ''
-    _toggle_post_message = ''
+    _toggle_pre_message = ""
+    _toggle_post_message = ""
 
     def toggle_server_extension(self, import_name):
         """Change the status of a named server extension.
@@ -216,8 +242,7 @@ class ToggleServerExtensionApp(BaseExtensionApp):
         """
         # Create an extension manager for this instance.
         config_dir, extension_manager = _get_extmanager_for_context(
-            user=self.user,
-            sys_prefix=self.sys_prefix
+            user=self.user, sys_prefix=self.sys_prefix
         )
         try:
             self.log.info("{}: {}".format(self._toggle_pre_message.capitalize(), import_name))
@@ -245,13 +270,14 @@ class ToggleServerExtensionApp(BaseExtensionApp):
     def start(self):
         """Perform the App's actions as configured"""
         if not self.extra_args:
-            sys.exit('Please specify a server extension/package to enable or disable')
+            sys.exit("Please specify a server extension/package to enable or disable")
         for arg in self.extra_args:
             self.toggle_server_extension(arg)
 
 
 class EnableServerExtensionApp(ToggleServerExtensionApp):
     """An App that enables (and validates) Server Extensions"""
+
     name = "jupyter server extension enable"
     description = """
     Enable a server extension in configuration.
@@ -266,6 +292,7 @@ class EnableServerExtensionApp(ToggleServerExtensionApp):
 
 class DisableServerExtensionApp(ToggleServerExtensionApp):
     """An App that disables Server Extensions"""
+
     name = "jupyter server extension disable"
     description = """
     Disable a server extension in configuration.
@@ -280,6 +307,7 @@ class DisableServerExtensionApp(ToggleServerExtensionApp):
 
 class ListServerExtensionsApp(BaseExtensionApp):
     """An App that lists (and validates) Server Extensions"""
+
     name = "jupyter server extension list"
     version = __version__
     description = "List all server extensions known by the configuration system"
@@ -292,7 +320,7 @@ class ListServerExtensionsApp(BaseExtensionApp):
         configurations = (
             {"user": True, "sys_prefix": False},
             {"user": False, "sys_prefix": True},
-            {"user": False, "sys_prefix": False}
+            {"user": False, "sys_prefix": False},
         )
 
         for option in configurations:
@@ -301,17 +329,13 @@ class ListServerExtensionsApp(BaseExtensionApp):
             for name, extension in ext_manager.extensions.items():
                 enabled = extension.enabled
                 # Attempt to get extension metadata
-                self.log.info(u'    {} {}'.format(
-                                name,
-                                GREEN_ENABLED if enabled else RED_DISABLED))
+                self.log.info("    {} {}".format(name, GREEN_ENABLED if enabled else RED_DISABLED))
                 try:
                     self.log.info("    - Validating {}...".format(name))
                     if not extension.validate():
                         raise ValueError("validation failed")
                     version = extension.version
-                    self.log.info(
-                        "      {} {} {}".format(name, version, GREEN_OK)
-                    )
+                    self.log.info("      {} {} {}".format(name, version, GREEN_OK))
                 except Exception as err:
                     self.log.warn("      {} {}".format(RED_X, err))
             # Add a blank line between paths.
@@ -331,6 +355,7 @@ jupyter server extension disable --py <packagename>  # disable all server extens
 
 class ServerExtensionApp(BaseExtensionApp):
     """Root level server extension app"""
+
     name = "jupyter server extension"
     version = __version__
     description = "Work with Jupyter server extensions"
@@ -339,7 +364,7 @@ class ServerExtensionApp(BaseExtensionApp):
     subcommands = dict(
         enable=(EnableServerExtensionApp, "Enable a server extension"),
         disable=(DisableServerExtensionApp, "Disable a server extension"),
-        list=(ListServerExtensionsApp, "List server extensions")
+        list=(ListServerExtensionsApp, "List server extensions"),
     )
 
     def start(self):
@@ -355,5 +380,5 @@ class ServerExtensionApp(BaseExtensionApp):
 main = ServerExtensionApp.launch_instance
 
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     main()
