@@ -1,37 +1,37 @@
 """
 Utilities for file-based Contents/Checkpoints managers.
 """
-
 # Copyright (c) Jupyter Development Team.
 # Distributed under the terms of the Modified BSD License.
-
-from contextlib import contextmanager
 import errno
 import io
 import os
 import shutil
+from base64 import decodebytes
+from base64 import encodebytes
+from contextlib import contextmanager
+from functools import partial
 
-from tornado.web import HTTPError
-
-from jupyter_server.utils import (
-    to_api_path,
-    to_os_path,
-)
 import nbformat
-
-from ipython_genutils.py3compat import str_to_unicode
-
-from traitlets.config import Configurable
+from anyio.to_thread import run_sync
+from tornado.web import HTTPError
 from traitlets import Bool
+from traitlets.config import Configurable
 
-from base64 import encodebytes, decodebytes
+from jupyter_server.utils import to_api_path
+from jupyter_server.utils import to_os_path
 
 
 def replace_file(src, dst):
-    """ replace dst with src
-    """
+    """replace dst with src"""
     os.replace(src, dst)
 
+
+async def async_replace_file(src, dst):
+    """replace dst with src asynchronously"""
+    await run_sync(os.replace, src, dst)
+
+
 def copy2_safe(src, dst, log=None):
     """copy src to dst
 
@@ -44,20 +44,36 @@ def copy2_safe(src, dst, log=None):
         if log:
             log.debug("copystat on %s failed", dst, exc_info=True)
 
-def path_to_intermediate(path):
-    '''Name of the intermediate file used in atomic writes.
 
-    The .~ prefix will make Dropbox ignore the temporary file.'''
+async def async_copy2_safe(src, dst, log=None):
+    """copy src to dst asynchronously
+
+    like shutil.copy2, but log errors in copystat instead of raising
+    """
+    await run_sync(shutil.copyfile, src, dst)
+    try:
+        await run_sync(shutil.copystat, src, dst)
+    except OSError:
+        if log:
+            log.debug("copystat on %s failed", dst, exc_info=True)
+
+
+def path_to_intermediate(path):
+    """Name of the intermediate file used in atomic writes.
+
+    The .~ prefix will make Dropbox ignore the temporary file."""
     dirname, basename = os.path.split(path)
-    return os.path.join(dirname, '.~'+basename)
+    return os.path.join(dirname, ".~" + basename)
+
 
 def path_to_invalid(path):
-    '''Name of invalid file after a failed atomic write and subsequent read.'''
+    """Name of invalid file after a failed atomic write and subsequent read."""
     dirname, basename = os.path.split(path)
-    return os.path.join(dirname, basename+'.invalid')
+    return os.path.join(dirname, basename + ".invalid")
+
 
 @contextmanager
-def atomic_writing(path, text=True, encoding='utf-8', log=None, **kwargs):
+def atomic_writing(path, text=True, encoding="utf-8", log=None, **kwargs):
     """Context manager to write to a file only if the entire write is successful.
 
     This works by copying the previous file contents to a temporary file in the
@@ -68,17 +84,14 @@ def atomic_writing(path, text=True, encoding='utf-8', log=None, **kwargs):
     Parameters
     ----------
     path : str
-      The target file to write to.
-
+        The target file to write to.
     text : bool, optional
-      Whether to open the file in text mode (i.e. to write unicode). Default is
-      True.
-
+        Whether to open the file in text mode (i.e. to write unicode). Default is
+        True.
     encoding : str, optional
-      The encoding to use for files opened in text mode. Default is UTF-8.
-
+        The encoding to use for files opened in text mode. Default is UTF-8.
     **kwargs
-      Passed to :func:`io.open`.
+        Passed to :func:`io.open`.
     """
     # realpath doesn't work on Windows: https://bugs.python.org/issue9949
     # Luckily, we only need to resolve the file itself being a symlink, not
@@ -93,10 +106,10 @@ def atomic_writing(path, text=True, encoding='utf-8', log=None, **kwargs):
 
     if text:
         # Make sure that text files have Unix linefeeds by default
-        kwargs.setdefault('newline', '\n')
-        fileobj = io.open(path, 'w', encoding=encoding, **kwargs)
+        kwargs.setdefault("newline", "\n")
+        fileobj = io.open(path, "w", encoding=encoding, **kwargs)
     else:
-        fileobj = io.open(path, 'wb', **kwargs)
+        fileobj = io.open(path, "wb", **kwargs)
 
     try:
         yield fileobj
@@ -116,26 +129,22 @@ def atomic_writing(path, text=True, encoding='utf-8', log=None, **kwargs):
         os.remove(tmp_path)
 
 
-
 @contextmanager
-def _simple_writing(path, text=True, encoding='utf-8', log=None, **kwargs):
+def _simple_writing(path, text=True, encoding="utf-8", log=None, **kwargs):
     """Context manager to write file without doing atomic writing
-    ( for weird filesystem eg: nfs).
+    (for weird filesystem eg: nfs).
 
     Parameters
     ----------
     path : str
-      The target file to write to.
-
+        The target file to write to.
     text : bool, optional
-      Whether to open the file in text mode (i.e. to write unicode). Default is
-      True.
-
+        Whether to open the file in text mode (i.e. to write unicode). Default is
+        True.
     encoding : str, optional
-      The encoding to use for files opened in text mode. Default is UTF-8.
-
+        The encoding to use for files opened in text mode. Default is UTF-8.
     **kwargs
-      Passed to :func:`io.open`.
+        Passed to :func:`io.open`.
     """
     # realpath doesn't work on Windows: https://bugs.python.org/issue9949
     # Luckily, we only need to resolve the file itself being a symlink, not
@@ -145,10 +154,10 @@ def _simple_writing(path, text=True, encoding='utf-8', log=None, **kwargs):
 
     if text:
         # Make sure that text files have Unix linefeeds by default
-        kwargs.setdefault('newline', '\n')
-        fileobj = io.open(path, 'w', encoding=encoding, **kwargs)
+        kwargs.setdefault("newline", "\n")
+        fileobj = io.open(path, "w", encoding=encoding, **kwargs)
     else:
-        fileobj = io.open(path, 'wb', **kwargs)
+        fileobj = io.open(path, "wb", **kwargs)
 
     try:
         yield fileobj
@@ -159,8 +168,6 @@ def _simple_writing(path, text=True, encoding='utf-8', log=None, **kwargs):
     fileobj.close()
 
 
-
-
 class FileManagerMixin(Configurable):
     """
     Mixin for ContentsAPI classes that interact with the filesystem.
@@ -179,10 +186,13 @@ class FileManagerMixin(Configurable):
     log : logging.Logger
     """
 
-    use_atomic_writing = Bool(True, config=True, help=
-    """By default notebooks are saved on disk on a temporary file and then if succefully written, it replaces the old ones.
+    use_atomic_writing = Bool(
+        True,
+        config=True,
+        help="""By default notebooks are saved on disk on a temporary file and then if succefully written, it replaces the old ones.
       This procedure, namely 'atomic_writing', causes some bugs on file system whitout operation order enforcement (like some networked fs).
-      If set to False, the new notebook is written directly on the old one which could fail (eg: full filesystem or quota )""")
+      If set to False, the new notebook is written directly on the old one which could fail (eg: full filesystem or quota )""",
+    )
 
     @contextmanager
     def open(self, os_path, *args, **kwargs):
@@ -205,7 +215,7 @@ class FileManagerMixin(Configurable):
                     yield f
 
     @contextmanager
-    def perm_to_403(self, os_path=''):
+    def perm_to_403(self, os_path=""):
         """context manager for turning permission errors into 403."""
         try:
             yield
@@ -215,9 +225,9 @@ class FileManagerMixin(Configurable):
                 # this may not work perfectly on unicode paths on Python 2,
                 # but nobody should be doing that anyway.
                 if not os_path:
-                    os_path = str_to_unicode(e.filename or 'unknown file')
+                    os_path = e.filename or "unknown file"
                 path = to_api_path(os_path, root=self.root_dir)
-                raise HTTPError(403, u'Permission denied: %s' % path) from e
+                raise HTTPError(403, "Permission denied: %s" % path) from e
             else:
                 raise
 
@@ -251,11 +261,13 @@ class FileManagerMixin(Configurable):
             raise HTTPError(404, "%s is outside root contents directory" % path)
         return os_path
 
-    def _read_notebook(self, os_path, as_version=4):
+    def _read_notebook(self, os_path, as_version=4, capture_validation_error=None):
         """Read a notebook from an os path."""
-        with self.open(os_path, 'r', encoding='utf-8') as f:
+        with self.open(os_path, "r", encoding="utf-8") as f:
             try:
-                return nbformat.read(f, as_version=as_version)
+                return nbformat.read(
+                    f, as_version=as_version, capture_validation_error=capture_validation_error
+                )
             except Exception as e:
                 e_orig = e
 
@@ -267,19 +279,26 @@ class FileManagerMixin(Configurable):
             if not self.use_atomic_writing or not os.path.exists(tmp_path):
                 raise HTTPError(
                     400,
-                    u"Unreadable Notebook: %s %r" % (os_path, e_orig),
+                    "Unreadable Notebook: %s %r" % (os_path, e_orig),
                 )
 
             # Move the bad file aside, restore the intermediate, and try again.
             invalid_file = path_to_invalid(os_path)
             replace_file(os_path, invalid_file)
             replace_file(tmp_path, os_path)
-            return self._read_notebook(os_path, as_version)
+            return self._read_notebook(
+                os_path, as_version, capture_validation_error=capture_validation_error
+            )
 
-    def _save_notebook(self, os_path, nb):
+    def _save_notebook(self, os_path, nb, capture_validation_error=None):
         """Save a notebook to an os_path."""
-        with self.atomic_writing(os_path, encoding='utf-8') as f:
-            nbformat.write(nb, f, version=nbformat.NO_CONVERT)
+        with self.atomic_writing(os_path, encoding="utf-8") as f:
+            nbformat.write(
+                nb,
+                f,
+                version=nbformat.NO_CONVERT,
+                capture_validation_error=capture_validation_error,
+            )
 
     def _read_file(self, os_path, format):
         """Read a non-notebook file.
@@ -293,40 +312,146 @@ class FileManagerMixin(Configurable):
         if not os.path.isfile(os_path):
             raise HTTPError(400, "Cannot read non-file %s" % os_path)
 
-        with self.open(os_path, 'rb') as f:
+        with self.open(os_path, "rb") as f:
             bcontent = f.read()
 
-        if format is None or format == 'text':
+        if format is None or format == "text":
             # Try to interpret as unicode if format is unknown or if unicode
             # was explicitly requested.
             try:
-                return bcontent.decode('utf8'), 'text'
+                return bcontent.decode("utf8"), "text"
             except UnicodeError as e:
-                if format == 'text':
+                if format == "text":
                     raise HTTPError(
                         400,
                         "%s is not UTF-8 encoded" % os_path,
-                        reason='bad format',
+                        reason="bad format",
                     ) from e
-        return encodebytes(bcontent).decode('ascii'), 'base64'
+        return encodebytes(bcontent).decode("ascii"), "base64"
 
     def _save_file(self, os_path, content, format):
         """Save content of a generic file."""
-        if format not in {'text', 'base64'}:
+        if format not in {"text", "base64"}:
             raise HTTPError(
                 400,
                 "Must specify format of file contents as 'text' or 'base64'",
             )
         try:
-            if format == 'text':
-                bcontent = content.encode('utf8')
+            if format == "text":
+                bcontent = content.encode("utf8")
             else:
-                b64_bytes = content.encode('ascii')
+                b64_bytes = content.encode("ascii")
                 bcontent = decodebytes(b64_bytes)
         except Exception as e:
-            raise HTTPError(
-                400, u'Encoding error saving %s: %s' % (os_path, e)
-            ) from e
+            raise HTTPError(400, "Encoding error saving %s: %s" % (os_path, e)) from e
 
         with self.atomic_writing(os_path, text=False) as f:
             f.write(bcontent)
+
+
+class AsyncFileManagerMixin(FileManagerMixin):
+    """
+    Mixin for ContentsAPI classes that interact with the filesystem asynchronously.
+    """
+
+    async def _copy(self, src, dest):
+        """copy src to dest
+
+        like shutil.copy2, but log errors in copystat
+        """
+        await async_copy2_safe(src, dest, log=self.log)
+
+    async def _read_notebook(self, os_path, as_version=4, capture_validation_error=None):
+        """Read a notebook from an os path."""
+        with self.open(os_path, "r", encoding="utf-8") as f:
+            try:
+                return await run_sync(
+                    partial(
+                        nbformat.read,
+                        as_version=as_version,
+                        capture_validation_error=capture_validation_error,
+                    ),
+                    f,
+                )
+            except Exception as e:
+                e_orig = e
+
+            # If use_atomic_writing is enabled, we'll guess that it was also
+            # enabled when this notebook was written and look for a valid
+            # atomic intermediate.
+            tmp_path = path_to_intermediate(os_path)
+
+            if not self.use_atomic_writing or not os.path.exists(tmp_path):
+                raise HTTPError(
+                    400,
+                    "Unreadable Notebook: %s %r" % (os_path, e_orig),
+                )
+
+            # Move the bad file aside, restore the intermediate, and try again.
+            invalid_file = path_to_invalid(os_path)
+            await async_replace_file(os_path, invalid_file)
+            await async_replace_file(tmp_path, os_path)
+            return await self._read_notebook(
+                os_path, as_version, capture_validation_error=capture_validation_error
+            )
+
+    async def _save_notebook(self, os_path, nb, capture_validation_error=None):
+        """Save a notebook to an os_path."""
+        with self.atomic_writing(os_path, encoding="utf-8") as f:
+            await run_sync(
+                partial(
+                    nbformat.write,
+                    version=nbformat.NO_CONVERT,
+                    capture_validation_error=capture_validation_error,
+                ),
+                nb,
+                f,
+            )
+
+    async def _read_file(self, os_path, format):
+        """Read a non-notebook file.
+
+        os_path: The path to be read.
+        format:
+          If 'text', the contents will be decoded as UTF-8.
+          If 'base64', the raw bytes contents will be encoded as base64.
+          If not specified, try to decode as UTF-8, and fall back to base64
+        """
+        if not os.path.isfile(os_path):
+            raise HTTPError(400, "Cannot read non-file %s" % os_path)
+
+        with self.open(os_path, "rb") as f:
+            bcontent = await run_sync(f.read)
+
+        if format is None or format == "text":
+            # Try to interpret as unicode if format is unknown or if unicode
+            # was explicitly requested.
+            try:
+                return bcontent.decode("utf8"), "text"
+            except UnicodeError as e:
+                if format == "text":
+                    raise HTTPError(
+                        400,
+                        "%s is not UTF-8 encoded" % os_path,
+                        reason="bad format",
+                    ) from e
+        return encodebytes(bcontent).decode("ascii"), "base64"
+
+    async def _save_file(self, os_path, content, format):
+        """Save content of a generic file."""
+        if format not in {"text", "base64"}:
+            raise HTTPError(
+                400,
+                "Must specify format of file contents as 'text' or 'base64'",
+            )
+        try:
+            if format == "text":
+                bcontent = content.encode("utf8")
+            else:
+                b64_bytes = content.encode("ascii")
+                bcontent = decodebytes(b64_bytes)
+        except Exception as e:
+            raise HTTPError(400, "Encoding error saving %s: %s" % (os_path, e)) from e
+
+        with self.atomic_writing(os_path, text=False) as f:
+            await run_sync(f.write, bcontent)
