@@ -11,13 +11,11 @@ from datetime import datetime
 
 import nbformat
 from anyio.to_thread import run_sync
-from ipython_genutils.importstring import import_item
 from jupyter_core.paths import exists
 from jupyter_core.paths import is_file_hidden
 from jupyter_core.paths import is_hidden
 from send2trash import send2trash
 from tornado import web
-from traitlets import Any
 from traitlets import Bool
 from traitlets import default
 from traitlets import TraitError
@@ -54,48 +52,6 @@ class FileContentsManager(FileManagerMixin, ContentsManager):
         except AttributeError:
             return os.getcwd()
 
-    post_save_hook = Any(
-        None,
-        config=True,
-        allow_none=True,
-        help="""Python callable or importstring thereof
-
-        to be called on the path of a file just saved.
-
-        This can be used to process the file on disk,
-        such as converting the notebook to a script or HTML via nbconvert.
-
-        It will be called as (all arguments passed by keyword)::
-
-            hook(os_path=os_path, model=model, contents_manager=instance)
-
-        - path: the filesystem path to the file just written
-        - model: the model representing the file
-        - contents_manager: this ContentsManager instance
-        """,
-    )
-
-    @validate("post_save_hook")
-    def _validate_post_save_hook(self, proposal):
-        value = proposal["value"]
-        if isinstance(value, str):
-            value = import_item(value)
-        if not callable(value):
-            raise TraitError("post_save_hook must be callable")
-        return value
-
-    def run_post_save_hook(self, model, os_path):
-        """Run the post-save hook if defined, and log errors"""
-        if self.post_save_hook:
-            try:
-                self.log.debug("Running post-save hook on %s", os_path)
-                self.post_save_hook(os_path=os_path, model=model, contents_manager=self)
-            except Exception as e:
-                self.log.error("Post-save hook failed o-n %s", os_path, exc_info=True)
-                raise web.HTTPError(
-                    500, u"Unexpected error while running post hook save: %s" % e
-                ) from e
-
     @validate("root_dir")
     def _validate_root_dir(self, proposal):
         """Do a bit of validation of the root_dir."""
@@ -155,6 +111,28 @@ class FileContentsManager(FileManagerMixin, ContentsManager):
         os_path = self._get_os_path(path=path)
         return is_hidden(os_path, self.root_dir)
 
+    def is_writable(self, path):
+        """Does the API style path correspond to a writable directory or file?
+
+        Parameters
+        ----------
+        path : string
+            The path to check. This is an API path (`/` separated,
+            relative to root_dir).
+
+        Returns
+        -------
+        hidden : bool
+            Whether the path exists and is writable.
+        """
+        path = path.strip("/")
+        os_path = self._get_os_path(path=path)
+        try:
+            return os.access(os_path, os.W_OK)
+        except OSError:
+            self.log.error("Failed to check write permissions on %s", os_path)
+            return False
+
     def file_exists(self, path):
         """Returns True if the file exists, else returns False.
 
@@ -251,12 +229,8 @@ class FileContentsManager(FileManagerMixin, ContentsManager):
         model["format"] = None
         model["mimetype"] = None
         model["size"] = size
+        model["writable"] = self.is_writable(path)
 
-        try:
-            model["writable"] = os.access(os_path, os.W_OK)
-        except OSError:
-            self.log.error("Failed to check write permissions on %s", os_path)
-            model["writable"] = False
         return model
 
     def _dir_model(self, path, content=True):
@@ -266,7 +240,7 @@ class FileContentsManager(FileManagerMixin, ContentsManager):
         """
         os_path = self._get_os_path(path)
 
-        four_o_four = u"directory does not exist: %r" % path
+        four_o_four = "directory does not exist: %r" % path
 
         if not os.path.isdir(os_path):
             raise web.HTTPError(404, four_o_four)
@@ -365,11 +339,14 @@ class FileContentsManager(FileManagerMixin, ContentsManager):
         os_path = self._get_os_path(path)
 
         if content:
-            nb = self._read_notebook(os_path, as_version=4)
+            validation_error = {}
+            nb = self._read_notebook(
+                os_path, as_version=4, capture_validation_error=validation_error
+            )
             self.mark_trusted_cells(nb, path)
             model["content"] = nb
             model["format"] = "json"
-            self.validate_notebook_model(model)
+            self.validate_notebook_model(model, validation_error)
 
         return model
 
@@ -398,14 +375,14 @@ class FileContentsManager(FileManagerMixin, ContentsManager):
         path = path.strip("/")
 
         if not self.exists(path):
-            raise web.HTTPError(404, u"No such file or directory: %s" % path)
+            raise web.HTTPError(404, "No such file or directory: %s" % path)
 
         os_path = self._get_os_path(path)
         if os.path.isdir(os_path):
             if type not in (None, "directory"):
                 raise web.HTTPError(
                     400,
-                    u"%s is a directory, not a %s" % (path, type),
+                    "%s is a directory, not a %s" % (path, type),
                     reason="bad type",
                 )
             model = self._dir_model(path, content=content)
@@ -413,19 +390,19 @@ class FileContentsManager(FileManagerMixin, ContentsManager):
             model = self._notebook_model(path, content=content)
         else:
             if type == "directory":
-                raise web.HTTPError(400, u"%s is not a directory" % path, reason="bad type")
+                raise web.HTTPError(400, "%s is not a directory" % path, reason="bad type")
             model = self._file_model(path, content=content, format=format)
         return model
 
     def _save_directory(self, os_path, model, path=""):
         """create a directory"""
         if is_hidden(os_path, self.root_dir) and not self.allow_hidden:
-            raise web.HTTPError(400, u"Cannot create hidden directory %r" % os_path)
+            raise web.HTTPError(400, "Cannot create hidden directory %r" % os_path)
         if not os.path.exists(os_path):
             with self.perm_to_403():
                 os.mkdir(os_path)
         elif not os.path.isdir(os_path):
-            raise web.HTTPError(400, u"Not a directory: %s" % (os_path))
+            raise web.HTTPError(400, "Not a directory: %s" % (os_path))
         else:
             self.log.debug("Directory %r already exists", os_path)
 
@@ -433,21 +410,22 @@ class FileContentsManager(FileManagerMixin, ContentsManager):
         """Save the file model and return the model with no content."""
         path = path.strip("/")
 
-        self.run_pre_save_hook(model=model, path=path)
+        self.run_pre_save_hooks(model=model, path=path)
 
         if "type" not in model:
-            raise web.HTTPError(400, u"No file type provided")
+            raise web.HTTPError(400, "No file type provided")
         if "content" not in model and model["type"] != "directory":
-            raise web.HTTPError(400, u"No file content provided")
+            raise web.HTTPError(400, "No file content provided")
 
         os_path = self._get_os_path(path)
         self.log.debug("Saving %s", os_path)
 
+        validation_error = {}
         try:
             if model["type"] == "notebook":
                 nb = nbformat.from_dict(model["content"])
                 self.check_and_sign(nb, path)
-                self._save_notebook(os_path, nb)
+                self._save_notebook(os_path, nb, capture_validation_error=validation_error)
                 # One checkpoint should always exist for notebooks.
                 if not self.checkpoints.list_checkpoints(path):
                     self.create_checkpoint(path)
@@ -461,21 +439,19 @@ class FileContentsManager(FileManagerMixin, ContentsManager):
         except web.HTTPError:
             raise
         except Exception as e:
-            self.log.error(u"Error while saving file: %s %s", path, e, exc_info=True)
-            raise web.HTTPError(
-                500, u"Unexpected error while saving file: %s %s" % (path, e)
-            ) from e
+            self.log.error("Error while saving file: %s %s", path, e, exc_info=True)
+            raise web.HTTPError(500, "Unexpected error while saving file: %s %s" % (path, e)) from e
 
         validation_message = None
         if model["type"] == "notebook":
-            self.validate_notebook_model(model)
+            self.validate_notebook_model(model, validation_error=validation_error)
             validation_message = model.get("message", None)
 
         model = self.get(path, content=False)
         if validation_message:
             model["message"] = validation_message
 
-        self.run_post_save_hook(model=model, os_path=os_path)
+        self.run_post_save_hooks(model=model, os_path=os_path)
 
         return model
 
@@ -485,7 +461,7 @@ class FileContentsManager(FileManagerMixin, ContentsManager):
         os_path = self._get_os_path(path)
         rm = os.unlink
         if not os.path.exists(os_path):
-            raise web.HTTPError(404, u"File or directory does not exist: %s" % os_path)
+            raise web.HTTPError(404, "File or directory does not exist: %s" % os_path)
 
         def _check_trash(os_path):
             if sys.platform in {"win32", "darwin"}:
@@ -512,12 +488,14 @@ class FileContentsManager(FileManagerMixin, ContentsManager):
             if not self.always_delete_dir and sys.platform == "win32" and is_non_empty_dir(os_path):
                 # send2trash can really delete files on Windows, so disallow
                 # deleting non-empty files. See Github issue 3631.
-                raise web.HTTPError(400, u"Directory %s not empty" % os_path)
+                raise web.HTTPError(400, "Directory %s not empty" % os_path)
             if _check_trash(os_path):
-                self.log.debug("Sending %s to trash", os_path)
                 # Looking at the code in send2trash, I don't think the errors it
                 # raises let us distinguish permission errors from other errors in
-                # code. So for now, just let them all get logged as server errors.
+                # code. So for now, the "look before you leap" approach is used.
+                if not self.is_writable(path):
+                    raise web.HTTPError(403, "Permission denied: %s" % path)
+                self.log.debug("Sending %s to trash", os_path)
                 send2trash(os_path)
                 return
             else:
@@ -529,7 +507,7 @@ class FileContentsManager(FileManagerMixin, ContentsManager):
         if os.path.isdir(os_path):
             # Don't permanently delete non-empty directories.
             if not self.always_delete_dir and is_non_empty_dir(os_path):
-                raise web.HTTPError(400, u"Directory %s not empty" % os_path)
+                raise web.HTTPError(400, "Directory %s not empty" % os_path)
             self.log.debug("Removing directory %s", os_path)
             with self.perm_to_403():
                 shutil.rmtree(os_path)
@@ -550,7 +528,7 @@ class FileContentsManager(FileManagerMixin, ContentsManager):
 
         # Should we proceed with the move?
         if os.path.exists(new_os_path) and not samefile(old_os_path, new_os_path):
-            raise web.HTTPError(409, u"File already exists: %s" % new_path)
+            raise web.HTTPError(409, "File already exists: %s" % new_path)
 
         # Move the file
         try:
@@ -559,7 +537,7 @@ class FileContentsManager(FileManagerMixin, ContentsManager):
         except web.HTTPError:
             raise
         except Exception as e:
-            raise web.HTTPError(500, u"Unknown error renaming file: %s %s" % (old_path, e)) from e
+            raise web.HTTPError(500, "Unknown error renaming file: %s %s" % (old_path, e)) from e
 
     def info_string(self):
         return _i18n("Serving notebooks from local directory: %s") % self.root_dir
@@ -587,7 +565,7 @@ class AsyncFileContentsManager(FileContentsManager, AsyncFileManagerMixin, Async
         """
         os_path = self._get_os_path(path)
 
-        four_o_four = u"directory does not exist: %r" % path
+        four_o_four = "directory does not exist: %r" % path
 
         if not os.path.isdir(os_path):
             raise web.HTTPError(404, four_o_four)
@@ -689,11 +667,14 @@ class AsyncFileContentsManager(FileContentsManager, AsyncFileManagerMixin, Async
         os_path = self._get_os_path(path)
 
         if content:
-            nb = await self._read_notebook(os_path, as_version=4)
+            validation_error = {}
+            nb = await self._read_notebook(
+                os_path, as_version=4, capture_validation_error=validation_error
+            )
             self.mark_trusted_cells(nb, path)
             model["content"] = nb
             model["format"] = "json"
-            self.validate_notebook_model(model)
+            self.validate_notebook_model(model, validation_error)
 
         return model
 
@@ -722,14 +703,14 @@ class AsyncFileContentsManager(FileContentsManager, AsyncFileManagerMixin, Async
         path = path.strip("/")
 
         if not self.exists(path):
-            raise web.HTTPError(404, u"No such file or directory: %s" % path)
+            raise web.HTTPError(404, "No such file or directory: %s" % path)
 
         os_path = self._get_os_path(path)
         if os.path.isdir(os_path):
             if type not in (None, "directory"):
                 raise web.HTTPError(
                     400,
-                    u"%s is a directory, not a %s" % (path, type),
+                    "%s is a directory, not a %s" % (path, type),
                     reason="bad type",
                 )
             model = await self._dir_model(path, content=content)
@@ -737,19 +718,19 @@ class AsyncFileContentsManager(FileContentsManager, AsyncFileManagerMixin, Async
             model = await self._notebook_model(path, content=content)
         else:
             if type == "directory":
-                raise web.HTTPError(400, u"%s is not a directory" % path, reason="bad type")
+                raise web.HTTPError(400, "%s is not a directory" % path, reason="bad type")
             model = await self._file_model(path, content=content, format=format)
         return model
 
     async def _save_directory(self, os_path, model, path=""):
         """create a directory"""
         if is_hidden(os_path, self.root_dir) and not self.allow_hidden:
-            raise web.HTTPError(400, u"Cannot create hidden directory %r" % os_path)
+            raise web.HTTPError(400, "Cannot create hidden directory %r" % os_path)
         if not os.path.exists(os_path):
             with self.perm_to_403():
                 await run_sync(os.mkdir, os_path)
         elif not os.path.isdir(os_path):
-            raise web.HTTPError(400, u"Not a directory: %s" % (os_path))
+            raise web.HTTPError(400, "Not a directory: %s" % (os_path))
         else:
             self.log.debug("Directory %r already exists", os_path)
 
@@ -757,20 +738,22 @@ class AsyncFileContentsManager(FileContentsManager, AsyncFileManagerMixin, Async
         """Save the file model and return the model with no content."""
         path = path.strip("/")
 
-        os_path = self._get_os_path(path)
-        self.log.debug("Saving %s", os_path)
         self.run_pre_save_hook(model=model, path=path)
 
         if "type" not in model:
-            raise web.HTTPError(400, u"No file type provided")
+            raise web.HTTPError(400, "No file type provided")
         if "content" not in model and model["type"] != "directory":
-            raise web.HTTPError(400, u"No file content provided")
+            raise web.HTTPError(400, "No file content provided")
 
+        os_path = self._get_os_path(path)
+        self.log.debug("Saving %s", os_path)
+
+        validation_error = {}
         try:
             if model["type"] == "notebook":
                 nb = nbformat.from_dict(model["content"])
                 self.check_and_sign(nb, path)
-                await self._save_notebook(os_path, nb)
+                await self._save_notebook(os_path, nb, capture_validation_error=validation_error)
                 # One checkpoint should always exist for notebooks.
                 if not (await self.checkpoints.list_checkpoints(path)):
                     await self.create_checkpoint(path)
@@ -784,21 +767,19 @@ class AsyncFileContentsManager(FileContentsManager, AsyncFileManagerMixin, Async
         except web.HTTPError:
             raise
         except Exception as e:
-            self.log.error(u"Error while saving file: %s %s", path, e, exc_info=True)
-            raise web.HTTPError(
-                500, u"Unexpected error while saving file: %s %s" % (path, e)
-            ) from e
+            self.log.error("Error while saving file: %s %s", path, e, exc_info=True)
+            raise web.HTTPError(500, "Unexpected error while saving file: %s %s" % (path, e)) from e
 
         validation_message = None
         if model["type"] == "notebook":
-            self.validate_notebook_model(model)
+            self.validate_notebook_model(model, validation_error=validation_error)
             validation_message = model.get("message", None)
 
         model = await self.get(path, content=False)
         if validation_message:
             model["message"] = validation_message
 
-        self.run_post_save_hook(model=model, os_path=os_path)
+        self.run_post_save_hooks(model=model, os_path=os_path)
 
         return model
 
@@ -808,7 +789,7 @@ class AsyncFileContentsManager(FileContentsManager, AsyncFileManagerMixin, Async
         os_path = self._get_os_path(path)
         rm = os.unlink
         if not os.path.exists(os_path):
-            raise web.HTTPError(404, u"File or directory does not exist: %s" % os_path)
+            raise web.HTTPError(404, "File or directory does not exist: %s" % os_path)
 
         async def _check_trash(os_path):
             if sys.platform in {"win32", "darwin"}:
@@ -840,12 +821,14 @@ class AsyncFileContentsManager(FileContentsManager, AsyncFileManagerMixin, Async
             ):
                 # send2trash can really delete files on Windows, so disallow
                 # deleting non-empty files. See Github issue 3631.
-                raise web.HTTPError(400, u"Directory %s not empty" % os_path)
+                raise web.HTTPError(400, "Directory %s not empty" % os_path)
             if await _check_trash(os_path):
-                self.log.debug("Sending %s to trash", os_path)
                 # Looking at the code in send2trash, I don't think the errors it
                 # raises let us distinguish permission errors from other errors in
-                # code. So for now, just let them all get logged as server errors.
+                # code. So for now, the "look before you leap" approach is used.
+                if not self.is_writable(path):
+                    raise web.HTTPError(403, "Permission denied: %s" % path)
+                self.log.debug("Sending %s to trash", os_path)
                 send2trash(os_path)
                 return
             else:
@@ -857,7 +840,7 @@ class AsyncFileContentsManager(FileContentsManager, AsyncFileManagerMixin, Async
         if os.path.isdir(os_path):
             # Don't permanently delete non-empty directories.
             if not self.always_delete_dir and await is_non_empty_dir(os_path):
-                raise web.HTTPError(400, u"Directory %s not empty" % os_path)
+                raise web.HTTPError(400, "Directory %s not empty" % os_path)
             self.log.debug("Removing directory %s", os_path)
             with self.perm_to_403():
                 await run_sync(shutil.rmtree, os_path)
@@ -878,7 +861,7 @@ class AsyncFileContentsManager(FileContentsManager, AsyncFileManagerMixin, Async
 
         # Should we proceed with the move?
         if os.path.exists(new_os_path) and not samefile(old_os_path, new_os_path):
-            raise web.HTTPError(409, u"File already exists: %s" % new_path)
+            raise web.HTTPError(409, "File already exists: %s" % new_path)
 
         # Move the file
         try:
@@ -887,4 +870,22 @@ class AsyncFileContentsManager(FileContentsManager, AsyncFileManagerMixin, Async
         except web.HTTPError:
             raise
         except Exception as e:
-            raise web.HTTPError(500, u"Unknown error renaming file: %s %s" % (old_path, e)) from e
+            raise web.HTTPError(500, "Unknown error renaming file: %s %s" % (old_path, e)) from e
+
+    async def dir_exists(self, path):
+        """Does a directory exist at the given path"""
+        path = path.strip("/")
+        os_path = self._get_os_path(path=path)
+        return os.path.isdir(os_path)
+
+    async def file_exists(self, path):
+        """Does a file exist at the given path"""
+        path = path.strip("/")
+        os_path = self._get_os_path(path)
+        return os.path.isfile(os_path)
+
+    async def is_hidden(self, path):
+        """Is path a hidden directory or file"""
+        path = path.strip("/")
+        os_path = self._get_os_path(path=path)
+        return is_hidden(os_path, self.root_dir)
